/**************************************************************
	LCD Serial communication monitor (inexpensive)
	 Sunlike SC2004(20 characters 4lines (80 character)) for PIC16F648A
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.52
	Tools:
	MPLABX ver 5.15
	XC8 ver 2.05
---------------------------------------------------------------
Date	:Time	Version	Build		Note
20081109:0700	1.00	---			begin the program
20081201:2100	1.01	---			Modification lcd.c
20081207:2145	1.10				J[\ǉALCD֌W̖̐wǂlcd.cɏW
20091117:0320	1.11				Fix lcd.c(v8.40/PICC pro9.70)
20190407:1417	1.50	5.15/2.05	Support new compiler and some bug fix and supported HEX mode
20190408:2355	1.51	5.15/2.05	fix communication buffer.
20190414:1700	1.52	5.15/2.05	Support CGRAM 
---------------------------------------------------------------
	Pin assignment
	1:RA2:O:LCD DB6
	2:RA3:O:LCD DB7
	3:RA4:I:GND
	4:MCLR:I:PICKIT2-1(W/10kPU)
	5:VSS:GND
	6:RB0:I:MODE SWITCH
	7:RB1:I:Recive serial
	8:RB2:O:Transmit serial(Not used)
	9:RB3:O:PWM out(Use negative potential)

	18:RA1:O:LCD DB5
	17:RA0:O:LCD DB4
	16:CLOCK:8MHz Resonator1
	15:CLOCK:8MHz Resonator2
	14:VDD:Power 3 to 5 Volt
	13:RB7:O:LCD E
	12:RB6:O:LCD RS
	11:RB5:I:RIGHT SW
	10:RB4:I:LEFT SW
---------------------------------------------------------------

ASCII[h
0x20`0xff̃R[hłΕ\

GXP[vV[PXŃJ[\ړ(PC̃J[\L[OK)
0x1b,0x5b,0x41@up
0x1b,0x5b,0x42 donw
0x1b,0x5b,0x43 right
0x1b,0x5b,0x44 left

ctrl+q J[\\
ctrl+w J[\\
ctrl+l LCDNA
ctrl+m J[\ȍ~s

ASCII[h
19200A38400bpsł63oCg𒴂AM
肱ڂ𔭐\܂B
 * J^JiSJIS[hőMĂB
Binary[h
38400bpsł63oCg𒴂AM͎肱ڂ𔭐܂B

 **************************************************************/

//#include <xc.h>		//***1.40
#include "main.h"	//Fix typo ***1.50
#include "sci.h"
#include "lcd.h"
#include "eeprom.h"	//***1.33
#include "promdata.h"	//***1.52

//interrupt handle

void __interrupt() my_isr_routine(void) { //Change to new notation ***1.50
	uint8_t temp;
	uint8_t data;
	//Handle RS232 Recieve interrupt
	if (RCIF == 1) {
		//recive to data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 Recieve interrupt bit
		if (FIFO_datasize < MAX_BUFF) {
			Buffer_FIFO[FIFO_write_pointer] = data;
			FIFO_write_pointer = (uint8_t) ((FIFO_write_pointer + 1) & (MAX_BUFF - 1)); //ring buff 
			FIFO_datasize++;
		}
	}
	if (TMR0IF == 1) { //timer interval interrupt
		if (!SW_LEFT) {
			while (!SW_LEFT); // button release check
			asm("RESET"); //reset
		}

		if (!SW_RIGHT) {
			temp = E2promRead(DisplayMode);
			if (temp != ASCIImode) E2promWrite(DisplayMode, ASCIImode);
			else E2promWrite(DisplayMode, HEXmode);
			while (!SW_RIGHT); // button release check
			asm("RESET"); //reset
		}

		if (sci_CheckOERR()) { //sci error
			CREN = 1;
		}

		//timer interval
		TMR0IF = 0;
	}
}

uint8_t Buffer_readdata(void) {
	uint8_t wdat;
	//read FIFO buffer
	while (FIFO_datasize == 0); // pause if buffer is empty
	wdat = Buffer_FIFO[FIFO_read_pointer];
	FIFO_read_pointer = (uint8_t) ((FIFO_read_pointer + 1)&(MAX_BUFF - 1)); //ring buff
	FIFO_datasize--;
	return wdat; //get wdat
	//	return FIFO_datasize;//debug
}

void Buffer_check(void) {
	uint8_t data;
	if (FIFO_datasize == MAX_BUFF - 1) {
		lcd_putch(0xFF);
		lcd_puts("Full");
		lcd_putch(0xFF);
		RCIE = 0; //terminal interrupt disable
		FIFO_datasize = 0; //FIFO clear
		FIFO_write_pointer = 0;
		FIFO_read_pointer = 0;
		CREN = 0; //Receive was aborted
		while (RCIF == 1) {
			data = sci_GetByte(); //Dummy read
		}
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		CREN = 1; //Receive is start
		RCIE = 1; //terminal interrupt enable				
	}
}

void delay_10ms(uint32_t temp) { //10ms multiple x times
	while (temp--) {
		__delay_ms(10);
	}
}


//Generate CG characters //***1.52
void lcd_CG_init(void) {
	int8_t temp, i, j, CGRAM_Address;
	CGRAM_Address = CGRAM_OffsetAddress;
	lcd_write(0x40+CGRAM_Address); //set CGRAM mode
	for (i = 0; i < 8; i++) {
		for (j = 0; j < 8; j++) {
			temp = E2promRead(CGRAM_Address++);
			lcd_cgwrite(temp);
		}
	}
	lcd_write(0x80);//DD RAM mode 
}


//------------------main

void main(void) {
	// Register
	uint32_t speed = 38400; //default communication speed set
	uint8_t temp, data, buffspeed, esc_flag = 0;
	//	Port initalize
	//		  	  76543210
	PORTA = 0b11100000; //Port Initalize
	LATA = 0b11100000; //Port Initalize
	PORTC = 0b00000000; //Port Initalize
	LATC = 0b00000000;
	TRISC = 0b11010000; //ccp1 output
	TRISA = 0b11111010; //
	ANSELA = 0x00; //All port are use for digital port
	ANSELC = 0x00; //All port are use for digital port
	CM1CON0 = 0x07; //PORTA is all digital I/O
	RXDTSEL = 1; //RA0 pin assignment as RXD
	OSCCON = 0b01110010; // PLL off,8MHz,IntOSC

	//Init
	OPTION_REG = 0b00000111; // prescale 1/256 RBPU= EN
	GIE = 0; //Int off

	//timer0 setting (interrupt event timer 3.328ms)
	TMR0IE = 0; //timer interrupt is off
	TMR0 = 0; //interrupt timer counter set

	//PWM initialize setting (83.8KHz on RB3 @8MHz)

	CCP1CON = 0b00001100; //Single output,LSBset,PWM mode: P1A, P1C active-high; P1B, P1D active-high
	PR2 = 0x17; // resolution is 6.5bit mode
	CCPR1L = 0x0c; // duty is 50%
	T2CON = 0b00000100; // POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1

	/*
	//PWM test (31.25KHz on RB3 @8MHz) test only
		CCP1CON = 0b00001100;// use PWM mode
		PR2 = 0xff; // resolution is 8bit mode
		CCPR1L = 0x20; // duty is 50%
		T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
	 */

	//LCD init
	//	DelayMs(200);	//stabilize time
	delay_10ms(20);
	lcd_init(); //lcd initalize
	lcd_clear(); //lcd clear

	//setting mode
	if (!SW_MODE) {
		lcd_write(LCD_CursorOff);
		lcd_goto(LCD_LineNumber1);
		lcd_puts(VersionInfo); //version kind of LCD
		lcd_puts(VersionCode); //version code
		while (!SW_MODE);
		//lcd_clear();
		//lcd_goto(LCD_LineNumber2);
		lcd_goto(LCD_LineNumber4);

#ifndef	_LCD0802
		lcd_puts("Set  DOWN  UP");
#else
		lcd_puts("Set -  +"); //**1.31
#endif	

		//Reading the last communication-speed setting
		buffspeed = E2promRead(COM_Speed);
		while (1) {
			__delay_ms(50); //DelayMs(50);
			lcd_goto(LCD_LineNumber3);
			switch (buffspeed) {
				case Speed1200:
					lcd_puts(" 120");
					break;
				case Speed2400:
					lcd_puts(" 240");
					break;
				case Speed4800:
					lcd_puts(" 480");
					break;
				case Speed9600:
					lcd_puts(" 960");
					break;
				case Speed19200:
					lcd_puts("1920");
					break;
				case Speed31250:
					lcd_puts("3125"); //***1.33
					break;
				case Speed38400:
					lcd_puts("3840");
					break;
				default: // data is broken
					buffspeed = Speed38400;
			}

			lcd_puts("0bps");

			//key check
			if (!SW_LEFT) {
				if (buffspeed > MIN_Speed) buffspeed--;
				while (!SW_LEFT);

			}
			if (!SW_RIGHT) {
				if (buffspeed < MAX_Speed) buffspeed++;
				while (!SW_RIGHT);

			}
			if (!SW_MODE) {
				if (buffspeed != E2promRead(COM_Speed)) {
					//Set New communication-speed
					E2promWrite(COM_Speed, buffspeed);
					lcd_goto(LCD_LineNumber2);
					lcd_puts("Setting has changed.");
					lcd_write(LCD_CursorBlink);
					//DelayMs(255);
					delay_10ms(200);
				}
				while (!SW_MODE);
				break;
			}
		}
	}

	//Reading the last communication-speed setting
	buffspeed = E2promRead(COM_Speed);
	//sci initialize
	switch (buffspeed) {
		case Speed1200:
			speed = 1200;
			break;
		case Speed2400:
			speed = 2400;
			break;
		case Speed4800:
			speed = 4800;
			break;
		case Speed9600:
			speed = 9600;
			break;
		case Speed19200:
			speed = 19200;
			break;
		case Speed31250: //***1.33
			speed = 31250;
			break;
		case Speed38400: //This speed is not accept HEX mode.
			speed = 38400;
			break;
		default:
			speed = 38400;
	}

	sci_Init(speed, SCI_EIGHT); // Baud set and Bit set

	//Interrupt enable
	//	INTEDG = 0; //RB0 Key interrupt fall edge
	//	INTE = 1;//RB0 is interrupt mode
	TMR0IE = 1; //timer interrupt on
	RCIE = 1; //recive interrupt on
	PEIE = 1; //Peripheral interrupt Enable 
	GIE = 1; //Global interrupt Enable 

	//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN = 1;
	}

	//Main routiene
	lcd_init();
	lcd_CG_init(); //***1.52
	lcd_clear(); //lcd clear


	/*
	//test
		temp = 0x20;
		while(1){
			if ( temp == 0xff ) temp = 0x20;
			lcd_putch(temp++);
			delay_10ms(20);
		}
		//test end	
	 */

	temp = E2promRead(DisplayMode);



	switch (temp) {
		case HEXmode:
			//ASCII to HEX convert(Use less or equal 19200bps)
			//lcd_puts("HEX:");
			//lcd_goto(0x40);
			lcd_write(LCD_CursorOn); //Init cursor
			while (1) {
				data = Buffer_readdata(); //Buffer read 

				//sprint format use
				//sprintf(outString, "%02x", data);
				//lcd_puts(outString);

				//Send high nibble
				if ((data >> 4) > 9)
					lcd_putch('A' - 10 + (uint8_t) (data >> 4));
				else lcd_putch('0' + (uint8_t) (data >> 4));
				//Send low nibble
				if ((data & 0x0F) > 9)
					lcd_putch('A' - 10 + (uint8_t) (data & 0x0F));
				else lcd_putch('0' + (uint8_t) (data & 0x0F));

				Buffer_check(); //Buffer check
			}
			break;

		default:
			//lcd_puts("ASCII:");
			//lcd_goto(0x40);
			lcd_write(LCD_CursorBlink); //Init cursor
			while (1) {
				data = Buffer_readdata(); //Buffer read 
				if (esc_flag == 1) {//esc flag check
					switch (data) {
						case 0x5b: //cursor control
							temp = Buffer_readdata(); //Buffer read again
							esc_flag = 0;
							switch (temp) {
								case 0x41: //up
									lcd_move(UP);
									break;

								case 0x42: //down
									lcd_move(DOWN);
									break;

								case 0x43: //right
									lcd_move(RIGHT);
									break;

								case 0x44: //left
									lcd_move(LEFT);
									break;

								default: //nothing
									break;
							}
							break;

						case 0x1b: //ESC sequence code
							esc_flag = 1;
							break;

						default: //nothing
							esc_flag = 0;
							break;
					}
				} else {
					switch (data) {
						case 0x1b: //ESC sequence code
							esc_flag = 1;
							break;

						case 12: //clear
							lcd_clear();
							break;

						case 13: //return command
							lcd_newline(1);
							break;

						case 0x11: //cursor off(ctrl +q)
							lcd_write(0x0c); //J[\\Ȃ
							break;

						case 0x17: //cursor on(ctrl +w)
							lcd_write(0x0f); //J[\\
							break;


						default:
							/*
							if 	(( data >= 0x20 ) && ( data <= 0xff  )) {
								sprintf(outString, "%c", data);//sprint use
								lcd_puts(outString);
							}
							 */
							
							//CG RAM display progress ***1.52
							if (data >= 0x20) {
								if ((data > 0x7F) && (data < 0x90)) {
									lcd_putch((uint8_t)(data - 0x80));
								} else {
									lcd_putch(data);
								}
							}
							break;
					}
				}
			}
			break;

	}

}
